﻿using System;
using System.IO.Ports;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace comGraph
{
    public partial class comGraph : Form
    {
        StringBuilder receivedLine = new StringBuilder();
        char[] delimiters = { ',', '\t' };
        Graphics gs, g0, g1, g2, g3;
        int[] chValue = { 0, 0, 0, 0 };
        bool SCALEis8, TXreqNow;
        string STX = "\x02";
        string ETX = "\x03";

        public comGraph()
        {
            InitializeComponent();
        }

        private void comGraph_Load(object sender, EventArgs e)
        {
            receivedLine.Length = 0;
            SCALEis8 = true;
            TXreqNow = false;

            Bitmap scaleBitmap = new Bitmap(
                pictureBox_scale.Size.Width, pictureBox_scale.Size.Height);
            pictureBox_scale.Image = scaleBitmap;
            gs = Graphics.FromImage(pictureBox_scale.Image);

            Bitmap ch0Bitmap = new Bitmap(pictureBox_ch0.Size.Width, pictureBox_ch0.Size.Height);
            pictureBox_ch0.Image = ch0Bitmap;
            g0 = Graphics.FromImage(pictureBox_ch0.Image);

            Bitmap ch1Bitmap = new Bitmap(pictureBox_ch1.Size.Width, pictureBox_ch1.Size.Height);
            pictureBox_ch1.Image = ch1Bitmap;
            g1 = Graphics.FromImage(pictureBox_ch1.Image);

            Bitmap ch2Bitmap = new Bitmap(pictureBox_ch2.Size.Width, pictureBox_ch2.Size.Height);
            pictureBox_ch2.Image = ch2Bitmap;
            g2 = Graphics.FromImage(pictureBox_ch2.Image);

            Bitmap ch3Bitmap = new Bitmap(pictureBox_ch3.Size.Width, pictureBox_ch3.Size.Height);
            pictureBox_ch3.Image = ch3Bitmap;
            g3 = Graphics.FromImage(pictureBox_ch3.Image);

            string[] ports = SerialPort.GetPortNames();
            foreach (string port in ports)
            {
                comboBox_port.Items.Add(port);
            }
            comboBox_port.SelectedItem = ports[0];
            comboBox_baudrate.SelectedItem = "9600";

            serialPort.Close();
            serialPort.NewLine = "\n";
            serialPort.PortName = ports[0];
            serialPort.BaudRate = Convert.ToInt32(comboBox_baudrate.SelectedItem);
            serialPort.Open();

            drawScale();
            drawLevel();
       }

        private void button1_Click(object sender, EventArgs e)
        {
            serialPort.Close();
            this.Close();
        }

        private void radioButton_8_CheckedChanged(object sender, EventArgs e)
        {
            SCALEis8 = true;
            drawScale();
            drawLevel();
        }

        private void radioButton_10_CheckedChanged(object sender, EventArgs e)
        {
            SCALEis8 = false;
            drawScale();
            drawLevel();
        }

        private void comboBox_port_SelectedIndexChanged(object sender, EventArgs e)
        {
            serialPort.Close();
            serialPort.PortName = comboBox_port.SelectedItem.ToString();
            serialPort.Open();
        }

        private void comboBox_baudRate_SelectedIndexChanged(object sender, EventArgs e)
        {
            serialPort.Close();
            serialPort.BaudRate = Convert.ToInt32(comboBox_baudrate.SelectedItem);
            serialPort.Open();
        }

        private void button_Exit_Click(object sender, EventArgs e)
        {
            serialPort.Close();
            this.Close();
        }

        private void button_sendRequest_Click(object sender, EventArgs e)
        {
            if (TXreqNow)
            {
                serialPort.Write(ETX);
                TXreqNow = false;
                button_Exit.Enabled = true;
                comboBox_port.Enabled = true;
                comboBox_baudrate.Enabled = true;
                button_sendRequest.Text = "送信要求";
            }
            else
            {
                serialPort.Write(STX);
                TXreqNow = true;
                button_Exit.Enabled = false;
                comboBox_port.Enabled = false;
                comboBox_baudrate.Enabled = false;
                button_sendRequest.Text = "停止要求";
            }
        }

        delegate void delegateMethod();

        private void serialPort_DataReceived(object sender, SerialDataReceivedEventArgs e)
        {
            try
            {
                int i = serialPort.BytesToRead;
                while (i-- > 0)
                {
                    int receivedData = serialPort.ReadByte();
                    if (receivedData == '\n')
                    {
                        setchValue();
                        receivedLine.Length = 0;
                        this.Invoke((delegateMethod)drawLevel, new object[] { });
                    }
                    else
                    {
                        receivedLine.Append((char)receivedData);
                    }
                }
            }
            catch
            {
                return;
            }
        }

        private void drawScale()
        {
            int rate;

            if (SCALEis8) rate = 2; else rate = 8;
            Font font = new Font("MS UI Gothic", 7);
            gs.Clear(pictureBox_scale.BackColor);
            gs.DrawLine(Pens.Black, 16, 0, 143, 0);
            for (int i = 0; i < 128; i += 5)
            {
                gs.DrawLine(Pens.Black, i + 16, 0, i + 16, 1);
            }

            for (int i = 0; i < 128; i += 25)
            {
                gs.DrawLine(Pens.Black, i + 16, 0, i + 16, 3);
                gs.DrawString(
                    String.Format("{0, 4}", i * rate),
                    font,
                    Brushes.Black, i + 3, 5
                    );
            }
            pictureBox_scale.Refresh();
        }

        private void setchValue()
        {
            string[] chWord = receivedLine.ToString().Split(delimiters);
            for (int i = 0; i < chValue.Length; i++)
            {
                if (i < chWord.Length)
                {
                    try {
                        if (chWord[i] == "") chValue[i] = 0;
                        else
                        chValue[i] = Convert.ToInt32(chWord[i]);
                    }
                    catch
                    {
                        chValue[i] = 0;
                    }
                }
                else
                {
                    chValue[i] = 0;
                }
            }
        }

        private void drawLevel()
        {
            int rate;
            if (SCALEis8) rate = 2; else rate = 8;

            g0.Clear(pictureBox_ch0.BackColor);
            g0.FillRectangle(Brushes.Silver, 0, 0, chValue[0] / rate, pictureBox_ch0.Size.Height);

            g1.Clear(pictureBox_ch1.BackColor);
            g1.FillRectangle(Brushes.Silver, 0, 0, chValue[1] / rate, pictureBox_ch1.Size.Height);

            g2.Clear(pictureBox_ch2.BackColor);
            g2.FillRectangle(Brushes.Silver, 0, 0, chValue[2] / rate, pictureBox_ch2.Size.Height);

            g3.Clear(pictureBox_ch3.BackColor);
            g3.FillRectangle(Brushes.Silver, 0, 0, chValue[3] / rate, pictureBox_ch3.Size.Height);

            label_ch0Value.Text = String.Format("{0, 4}", chValue[0]);
            label_ch1Value.Text = String.Format("{0, 4}", chValue[1]);
            label_ch2Value.Text = String.Format("{0, 4}", chValue[2]);
            label_ch3Value.Text = String.Format("{0, 4}", chValue[3]);

            pictureBox_ch0.Refresh();
            pictureBox_ch1.Refresh();
            pictureBox_ch2.Refresh();
            pictureBox_ch3.Refresh();
        }
    }
}